
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <string>
using std::string;

#include "refcount.h"

#include "BindObjectToName.h"

#include "NamedCounterFactory.h"

int main(int argc, char **argv)
{
   try 
   {
      size_t numCounters = 0;

      bool bDefaultCounters = false;

      if (argc > 1)
      {
         numCounters = atoi(argv[1]);
      }

	   CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");

      CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
      PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

      PortableServer::POAManager_var poa_man = poa->the_POAManager();
      poa_man->activate();

      if (0 == numCounters)
      {
         numCounters = 10;
         bDefaultCounters = true;
      }

      NamedCounterFactoryImpl *pFactory = new NamedCounterFactoryImpl(numCounters);

      if (bDefaultCounters)
      {
         cout << "Server - starting with " << numCounters << " counters" << endl;
         cout << "you can override the number of counters by passing a value" << endl;
         cout << "as the first command line parameter" << endl;
         cout << "e.g. server 3000" << endl;
      }

      poa->activate_object(pFactory);

      obj = pFactory->_this();
    
      if(!BindObjectToName(orb, obj, "JetByte Test", "NamedCounterFactory"))
      {
         cout << "Failed to bind object to name" << endl;
         return 1;
      }

      pFactory->_remove_ref();

      orb->run();
      orb->destroy();
  }
  catch(CORBA::SystemException &) 
  {
    cerr << "Caught CORBA::SystemException." << endl;
  }
  catch(CORBA::Exception&) 
  {
    cerr << "Caught CORBA::Exception." << endl;
  }
  catch(omniORB::fatalException& fe) 
  {
    cerr << "Caught omniORB::fatalException:" << endl;
    cerr << "  file: " << fe.file() << endl;
    cerr << "  line: " << fe.line() << endl;
    cerr << "  mesg: " << fe.errmsg() << endl;
  }
  catch(...) 
  {
    cerr << "Caught unknown exception." << endl;
  }

  return 0;
}

